#pragma once

// Eigen Ice definition
#include <ArmarXCore/interface/serialization/Eigen.ice>

// AronMacros
#include <RobotAPI/libraries/aron/AronConfig.h>

module armarx
{
    module aron
    {
        /*************************
         * General Definitions ***
         ************************/
        sequence<byte> AronBlobSequence;
        sequence<int> AronIntSequence;

        /*************************
         * Aron Types ************
         ************************/
        class AronAbstractType { };
        sequence<AronAbstractType> AronAbstractTypeList;
        dictionary<string, AronAbstractType> AronAbstractTypeDict;

        // External Type
        class AronExternalType extends AronAbstractType
        { };

        // Container Types
        class AronListType extends AronAbstractType { AronAbstractType acceptedType; };
        class AronObjectType extends AronAbstractType { string name; AronAbstractTypeDict elementTypes; };
        class AronDictType extends AronAbstractType { AronAbstractType acceptedType; };
        class AronTupleType extends AronAbstractType { AronAbstractTypeList elementTypes; };

        // Complex Types
        class AronEigenMatrixType extends AronAbstractType { AronIntSequence dimensions; string typeName; };
        class AronIVTCByteImageType extends AronAbstractType { int width; int height; string typeName; };
        class AronOpenCVMatType extends AronAbstractType { AronIntSequence dimensions; string typeName; };
        class AronPCLPointcloudType extends AronAbstractType { int width; int height; string typeName; };

        // Primitive Types
#define RUN_ARON_MACRO(cppType, upperType, lowerType, capsType) \
    class Aron##upperType##Type extends AronAbstractType { };

        HANDLE_PRIMITIVE_TYPES_WITH_CPP_TYPE
#undef RUN_ARON_MACRO


        /*************************
         * Aron Data *************
         ************************/
        class AronData { };
        sequence<AronData> AronDataList;
        dictionary<string, AronData> AronDataDict;

        // Container Data
        class AronList extends AronData { AronDataList elements; };
        class AronDict extends AronData { AronDataDict elements; };

        // Complex Data
        class AronBlob extends AronData { AronBlobSequence data; };

        // Basic Data
#define RUN_ARON_MACRO(cppType, upperType, lowerType, capsType) \
    class Aron##upperType extends AronData { cppType value; };

        HANDLE_PRIMITIVE_TYPES_WITH_CPP_TYPE
#undef RUN_ARON_MACRO

    };
};
