#include "MemoryComponentPlugin.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "../error.h"

#include "IceMemory.h"


namespace armarx::armem
{

    MemoryComponentPluginUser::MemoryComponentPluginUser()
    {
        addPlugin(plugin);
    }


    // WRITING

    armem::data::AddSegmentsResult MemoryComponentPluginUser::addSegments(const armem::data::AddSegmentsInput& input, const Ice::Current&)
    {
        bool addCoreSegmentOnUsage = false;
        return addSegments(input, addCoreSegmentOnUsage);
    }

    armem::data::AddSegmentsResult MemoryComponentPluginUser::addSegments(const armem::data::AddSegmentsInput& input, bool addCoreSegments)
    {
        std::scoped_lock lock(memoryMutex);
        armem::data::AddSegmentsResult result = iceMemory.addSegments(input, addCoreSegments);
        return result;
    }


    armem::data::CommitResult MemoryComponentPluginUser::commit(const armem::data::Commit& commitIce, const Ice::Current&)
    {
        std::scoped_lock lock(memoryMutex);
        return iceMemory.commit(commitIce);
    }



    // READING

    armem::data::EntitySnapshotQueryResultList MemoryComponentPluginUser::getEntitySnapshots(
        const armem::data::EntitySnapshotQueryList& queries, const Ice::Current&)
    {
        std::scoped_lock lock(memoryMutex);

        armem::data::EntitySnapshotQueryResultList results;

        for (const auto& query : queries)
        {
            armem::data::EntitySnapshotQueryResult& result = results.emplace_back();
            armem::MemoryID id = armem::MemoryID::fromString(query.snapshotID);

            try
            {
                armem::EntitySnapshot& snapshot = memory.getEntitySnapshot(id);
                result.success = true;
                result.timeCreatedMicroSeconds = snapshot.time.toMicroSeconds();
                for (const auto& instance : snapshot.instances)
                {
                    result.instances.push_back(instance->data);
                }
            }
            catch (const armem::error::ArMemError& e)
            {
                result.success = false;
                result.errorMessage = e.what();
            }
        }

        return results;
    }


}
