#pragma once

#include <mutex>

#include <ArmarXCore/core/Component.h>

#include <RobotAPI/interface/armem/MemoryInterface.h>

#include "../memory/Memory.h"
#include "IceMemory.h"


namespace armarx::armem::plugins
{

    class MemoryComponentPlugin : public ComponentPlugin
    {
    public:
        using ComponentPlugin::ComponentPlugin;

    };

}


namespace armarx::armem
{

    /**
     * @brief Utility for connecting a Memory to Ice.
     */
    class MemoryComponentPluginUser :
        virtual public ManagedIceObject
        , virtual public MemoryInterface
    {
    public:

        MemoryComponentPluginUser();


        // WritingInterface interface
        virtual armem::data::AddSegmentsResult addSegments(const armem::data::AddSegmentsInput& input, const Ice::Current& = Ice::emptyCurrent) override;
        armem::data::AddSegmentsResult addSegments(const armem::data::AddSegmentsInput& input, bool addCoreSegments);

        virtual armem::data::CommitResult commit(const armem::data::Commit& commit, const Ice::Current& = Ice::emptyCurrent) override;


        // ReadingInterface interface
        virtual armem::data::EntitySnapshotQueryResultList getEntitySnapshots(const armem::data::EntitySnapshotQueryList&, const Ice::Current& = Ice::emptyCurrent) override;


    public:

        std::mutex memoryMutex;

        /// The actual memory.
        Memory memory;

        /// Helps connecting `memory` to ice. Used to handle Ice callbacks.
        IceMemory iceMemory { &memory };


    private:

        armem::plugins::MemoryComponentPlugin* plugin = nullptr;


    };


}
