/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <unordered_map>

// ArmarX
#include <ArmarXCore/core/exceptions/Exception.h>

#include <RobotAPI/libraries/aron/AronConcepts.h>
#include <RobotAPI/libraries/aron/AronDescriptor.h>
#include <RobotAPI/libraries/aron/AronResolver.h>
#include <RobotAPI/libraries/aron/AronNavigatorPath.h>

namespace armarx
{
    namespace aron
    {
        template <typename Descriptor, typename Aron> //requires isAron<Aron> && isTypeDescriptor<Descriptor>
        class AronNavigator;

        template <typename Descriptor, typename Aron> //requires isAron<Aron> && isTypeDescriptor<Descriptor>
        using AronNavigatorPtr = std::shared_ptr<AronNavigator<Descriptor, Aron>>;

        template <typename Descriptor, typename Aron> //requires isAron<Aron> && isTypeDescriptor<Descriptor>
        class AronNavigator
        {
        public:
            // constructors
            AronNavigator(const Descriptor& descriptor) :
                descriptor(descriptor)
            {
            }

            // public member functions
            Descriptor getDescriptor() const
            {
                return descriptor;
            }

            // virual definitions
            virtual typename Aron::PointerType getResult() const = 0;

            // static methods
            template<typename AronNavigatorType> //requires isAronNavigator<AronNavigatorType>
            static typename AronNavigatorType::PointerType FromAronPtr(const typename Aron::PointerType& n)
            {
                return AronNavigatorType::DynamicCast(AronNavigatorType::FromAronPtr(n));
            }

        protected:
            static void checkAronPtrForNull(const typename Aron::PointerType& data, const std::string& message)
            {
                if (data.get() == nullptr)
                {
                    throw LocalException("Could not cast an AronPtr. The Ptr was NULL. " + message);
                }
            }

        private:
            // members
            const Descriptor descriptor;
        };
    }
}
