/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "AronNavigatorPath.h"

// ArmarX
#include <ArmarXCore/core/exceptions/Exception.h>


namespace armarx
{
    namespace aron
    {

        AronNavigatorPath::AronNavigatorPath() :
            delimeter("/")
        {

        }

        AronNavigatorPath::AronNavigatorPath(const std::string& d):
            delimeter(d)
        {

        }

        AronNavigatorPath::AronNavigatorPath(const AronNavigatorPath& p) :
            path(p.getPath())
        {

        }

        AronNavigatorPath::AronNavigatorPath(const AronNavigatorPath& p, const std::string& s) :
            AronNavigatorPath(p)
        {
            appendPath(s);
        }

        AronNavigatorPath::AronNavigatorPath(const AronNavigatorPath& p, const std::string& s, const std::string& s2) :
            AronNavigatorPath(p)
        {
            appendPath(s);
            appendPath(s2);
        }

        void AronNavigatorPath::appendPath(const std::string& str)
        {
            path.push_back(str);
        }

        std::vector<std::string> AronNavigatorPath::getPath() const
        {
            return path;
        }

        std::string AronNavigatorPath::getLastElement() const
        {
            if (!path.size())
            {
                throw LocalException("Try to access last element of empty vector.");
            }
            return path.back();
        }

        std::string AronNavigatorPath::toString() const
        {
            std::stringstream ss("/");
            for (const std::string& s : path)
            {
                ss << "->" << s;
            }
            return ss.str();
        }

    }
}
