/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Boost
#include <boost/core/demangle.hpp>

// Header
#include "AronResolver.h"

namespace armarx
{
    namespace aron
    {
        AronTypeDescriptor AronResolver::GetTypeForAronAbstractType(const AronAbstractTypePtr& a)
        {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    { \
        Aron##upperType##TypePtr aron = Aron##upperType##TypePtr::dynamicCast(a); \
        if(aron.get() != nullptr) \
        { \
            return eAron##upperType##Type; \
        } \
    }
            HANDLE_ALL_ARON_TYPES_WITHOUT_CPP_TYPE
#undef RUN_ARON_MACRO
            throw LocalException("AronTypeResolver: Could not cast AronAbstractType to a valid type.");
        }

        AronTypeDescriptor AronResolver::GetTypeForAronAbstractTypeId(const std::type_index& a)
        {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    { \
        if(a == std::type_index(typeid(Aron##upperType##Type))) \
        { \
            return eAron##upperType##Type; \
        } \
    }
            HANDLE_ALL_ARON_TYPES_WITHOUT_CPP_TYPE
#undef RUN_ARON_MACRO
            throw LocalException("AronTypeResolver: Could not cast AronAbstractTypeId to a valid type. The typeid was " + boost::core::demangle(a.name()));
        }

        AronDataDescriptor AronResolver::GetTypeForAronData(const AronDataPtr& a)
        {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    { \
        Aron##upperType##Ptr aron = Aron##upperType##Ptr::dynamicCast(a); \
        if(aron.get() != nullptr) \
        { \
            return eAron##upperType##Data; \
        } \
    }
            HANDLE_ALL_ARON_DATA_WITHOUT_CPP_TYPE
#undef RUN_ARON_MACRO
            throw LocalException("AronTypeResolver: Could not cast AronData to a valid type.");
        }

        AronDataDescriptor AronResolver::GetTypeForAronDataTypeId(const std::type_index& a)
        {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    { \
        if(a == std::type_index(typeid(Aron##upperType))) \
        { \
            return eAron##upperType##Data; \
        } \
    }
            HANDLE_ALL_ARON_DATA_WITHOUT_CPP_TYPE
#undef RUN_ARON_MACRO
            throw LocalException("AronTypeResolver: Could not cast AronDataTypeId to a valid type. The typeid was " + boost::core::demangle(a.name()));
        }
    }
}
