/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <set>
#include <vector>


// ArmarX
#include <ArmarXCore/libraries/cppgen/CppMethod.h>
#include <ArmarXCore/libraries/cppgen/CppClass.h>

#include <RobotAPI/libraries/aron/types/AronAbstractTypeNavigator.h>
#include <RobotAPI/libraries/aron/types/AronObjectTypeNavigator.h>
#include <RobotAPI/libraries/aron/types/AronListTypeNavigator.h>
#include <RobotAPI/libraries/aron/types/AronPrimitiveTypeNavigator.h>

#include <RobotAPI/libraries/aron/codegenerator/AronWriterInfo.h>
#include <RobotAPI/libraries/aron/codegenerator/AronReaderInfo.h>
#include <RobotAPI/libraries/aron/codegenerator/AronUseTypeInfo.h>

namespace armarx
{
    namespace aron
    {
        namespace codegeneration
        {
            namespace classwriter
            {
                class AronTypeClassCppWriter;
                typedef std::shared_ptr<AronTypeClassCppWriter> AronConnectorCppWriterPtr;

                class AronTypeClassCppWriter
                {
                public:
                    typedef std::vector<std::pair<types::AronObjectTypeNavigatorPtr, std::vector<types::AronObjectTypeNavigatorPtr>>> GenerateTypePtrList;

                    AronTypeClassCppWriter(const std::string&, const std::set<AronWriterInfo>&, const std::set<AronReaderInfo>&, const std::vector<AronUseTypeInfo>&, const std::vector<std::string>&);

                    void generateTypeClasses(const GenerateTypePtrList&, bool onlyAddHeaderInformationToFirstClass = false); // Todo true is not supported at the moment since the guard is missing
                    std::vector<CppClassPtr> getTypeClasses() const;

                private:
                    CppClassPtr setupCppClassFromObjectType(const types::AronObjectTypeNavigatorPtr&) const;
                    void setupIncludes(CppClassPtr&, const std::string&) const;

                private:
                    std::string producerName;
                    std::set<AronWriterInfo> writers;
                    std::set<AronReaderInfo> readers;
                    std::vector<std::string> additionalIncludes;
                    std::vector<CppClassPtr> typeClasses;
                };
            }
        }
    }
}
