/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <filesystem>

// ArmarX
#include <RobotAPI/libraries/aron/types/AronObjectTypeNavigator.h>
#include <RobotAPI/libraries/aron/codegenerator/AronReaderInfo.h>
#include <RobotAPI/libraries/aron/codegenerator/AronWriterInfo.h>
#include <RobotAPI/libraries/aron/codegenerator/AronUseTypeInfo.h>

namespace armarx
{
    namespace aron
    {
        namespace codegeneration
        {
            namespace typereader
            {
                template <typename Input>
                class AronAbstractTypeReader;

                template <typename Input>
                using AronAbstractTypeReaderPtr = std::shared_ptr<AronAbstractTypeReader<Input>>;

                template <typename Input>
                class AronAbstractTypeReader
                {
                public:
                    AronAbstractTypeReader()
                    {
                        // The toAron Serializer is visible by default
                        AronWriterInfo toAron;
                        toAron.methodName = "toAron";
                        toAron.returnType = "armarx::aron::AronDataPtr";
                        toAron.writerClassType = "armarx::aron::io::AronDataNavigatorWriter";
                        toAron.include = "<RobotAPI/libraries/aron/io/classWriters/AronDataNavigatorWriter/AronDataNavigatorWriter.h>";
                        writers.insert(toAron);

                        // The fromAron method is visible by deafult
                        AronReaderInfo fromAron;
                        fromAron.methodName = "fromAron";
                        fromAron.argumentType = "armarx::aron::AronDataPtr";
                        fromAron.readerClassType = "armarx::aron::io::AronDataNavigatorReader";
                        fromAron.include = "<RobotAPI/libraries/aron/io/classReaders/AronDataNavigatorReader/AronDataNavigatorReader.h>";
                        readers.insert(fromAron);
                    }

                    virtual void parse(const Input&) = 0;
                    virtual void parseFile(const std::string& filename) = 0;
                    virtual void parseFile(const std::filesystem::path& file) = 0;

                    std::vector<std::string> getAdditionalIncludes() const
                    {
                        return additionalIncludes;
                    }
                    std::set<AronWriterInfo> getWriters() const
                    {
                        return writers;
                    }
                    std::set<AronReaderInfo> getReaders() const
                    {
                        return readers;
                    }

                    std::vector<AronUseTypeInfo> getUseTypes() const
                    {
                        return useTypes;
                    }
                    std::vector<std::pair<types::AronObjectTypeNavigatorPtr, std::vector<types::AronObjectTypeNavigatorPtr>>> getGenerateTypes() const
                    {
                        return generateTypes;
                    }

                protected:
                    std::vector<std::string> additionalIncludes;
                    std::set<AronReaderInfo> readers;
                    std::set<AronWriterInfo> writers;
                    std::vector<AronUseTypeInfo> useTypes;
                    std::vector<std::pair<types::AronObjectTypeNavigatorPtr, std::vector<types::AronObjectTypeNavigatorPtr>>> generateTypes;
                };
            }
        }
    }
}
