/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <map>

// ArmarX
#include <ArmarXCore/core/rapidxml/wrapper/RapidXmlReader.h>

#include <RobotAPI/libraries/aron/AronConcepts.h>

#include <RobotAPI/libraries/aron/types/AronAbstractTypeNavigator.h>
#include <RobotAPI/libraries/aron/types/AronObjectTypeNavigator.h>
#include <RobotAPI/libraries/aron/types/AronListTypeNavigator.h>
#include <RobotAPI/libraries/aron/types/AronPrimitiveTypeNavigator.h>


namespace armarx
{
    namespace aron
    {
        namespace codegeneration
        {
            namespace typereader
            {
                class AronAbstractTypeXMLReaderData
                {
                public:
                    // Definition tags
                    const static std::string ARON_TYPE_DEFINITION_TAG;

                    const static std::string ARON_INCLUDES_TAG;
                    const static std::string ARON_SERIALIZERS_TAG;
                    const static std::string ARON_USE_TYPES_TAG;
                    const static std::string ARON_PRODUCE_TYPES_TAG;

                    const static std::string ARON_INCLUDE_TAG;
                    const static std::string ARON_WRITERS_TAG;
                    const static std::string ARON_READERS_TAG;

                    const static std::string ARON_WRITER_TAG;
                    const static std::string ARON_READER_TAG;
                    const static std::string ARON_USE_TYPE_TAG;
                    const static std::string ARON_PRODUCE_TYPE_TAG;

                    // Attribute names
                    const static std::string ARON_METHOD_ATTRIBUTE_NAME;
                    const static std::string ARON_RETURN_ATTRIBUTE_NAME;
                    const static std::string ARON_ARGUMENT_TYPE_ATTRIBUTE_NAME;
                    const static std::string ARON_INCLUDE_ATTRIBUTE_NAME;
                    const static std::string ARON_READER_ATTRIBUTE_NAME;
                    const static std::string ARON_WRITER_ATTRIBUTE_NAME;
                    const static std::string ARON_NAME_ATTRIBUTE_NAME;
                    const static std::string ARON_KEY_ATTRIBUTE_NAME;
                    const static std::string ARON_TYPE_ATTRIBUTE_NAME;
                    const static std::string ARON_WIDTH_ATTRIBUTE_NAME;
                    const static std::string ARON_HEIGHT_ATTRIBUTE_NAME;

                    // Second level tags. Only important if in specific top level tag
                    const static std::string ARON_OBJECT_CHILD_TAG;
                    const static std::string ARON_TYPE_TAG;
                    const static std::string ARON_DIMENSIONS_TAG;
                    const static std::string ARON_DIMENSION_TAG;

                    // Top Level type tags
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    const static std::string ARON_PRODUCE_##capsType##_TAG;

                    HANDLE_ALL_ARON_TYPES_WITHOUT_CPP_TYPE
#undef RUN_ARON_MACRO

                    static void EnforceAttribute(const RapidXmlReaderNode& node, const std::string& att);
                    static bool HasAttribute(const RapidXmlReaderNode& node, const std::string& att);
                    static bool HasTagName(const RapidXmlReaderNode& node, const std::string& name);
                    static void EnforceTagName(const RapidXmlReaderNode& node, const std::string& name);
                    static bool CheckMinChildSize(const RapidXmlReaderNode& node, const size_t size);
                    static bool CheckMaxChildSize(const RapidXmlReaderNode& node, const size_t size);
                    static bool CheckExactChildSize(const RapidXmlReaderNode& node, const size_t size);
                    static void EnforceChildSize(const RapidXmlReaderNode& node, const size_t size);
                };
            }
        }
    }
}
