/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <filesystem>

// Base Class
#include <RobotAPI/libraries/aron/codegenerator/typeReaders/AronAbstractTypeReader.h>

// ArmarX
#include <ArmarXCore/core/rapidxml/wrapper/RapidXmlReader.h>

#include <RobotAPI/libraries/aron/types/AronObjectTypeNavigator.h>
#include <RobotAPI/libraries/aron/codegenerator/typeReaders/xmlReader/AronAbstractTypeXMLReaderFactory.h>

namespace armarx
{
    namespace aron
    {
        namespace codegeneration
        {
            namespace typereader
            {
                class AronAbstractTypeXMLReader;
                typedef std::shared_ptr<AronAbstractTypeXMLReader> AronAbstractTypeXMLReaderPtr;

                class AronAbstractTypeXMLReader :
                    virtual public AronAbstractTypeReader<std::string>
                {
                public:
                    AronAbstractTypeXMLReader() = default;

                    virtual void parse(const std::string&) override;
                    virtual void parseFile(const std::string&) override;
                    virtual void parseFile(const std::filesystem::path&) override;

                private:
                    void parse(const RapidXmlReaderPtr& node);

                    std::string readInclude(const RapidXmlReaderNode& node) const;
                    AronWriterInfo readWriter(const RapidXmlReaderNode& node) const;
                    AronReaderInfo readReader(const RapidXmlReaderNode& node) const;
                    AronUseTypeInfo readUseType(const RapidXmlReaderNode& node) const;
                    types::AronObjectTypeNavigatorPtr readGenerateType(const RapidXmlReaderNode& node) const;

                private:
                    static const AronObjectTypeXMLReaderFactory Factory;
                };
            }
        }
    }
}
