/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>

// Base Class
#include <RobotAPI/libraries/aron/codegenerator/typeReaders/AronAbstractTypeReader.h>

// ArmarX
#include <ArmarXCore/core/rapidxml/wrapper/RapidXmlReader.h>

#include <RobotAPI/libraries/aron/AronFactory.h>
#include <RobotAPI/libraries/aron/types/AronAllTypeNavigators.h>

using namespace std;

namespace armarx
{
    namespace aron
    {
        namespace codegeneration
        {
            namespace typereader
            {
                class AronAbstractTypeXMLReaderFactory;
                typedef std::shared_ptr<AronAbstractTypeXMLReaderFactory> AronAbstractTypeXMLReaderFactoryPtr;

                class AronAbstractTypeXMLReaderFactory :
                    virtual public AronFactory<RapidXmlReaderNode, types::AronAbstractTypeNavigatorPtr>
                {
                public:
                    AronAbstractTypeXMLReaderFactory() = default;

                    virtual types::AronAbstractTypeNavigatorPtr create(const RapidXmlReaderNode&, const AronNavigatorPath&) const override;
                    virtual types::AronAbstractTypeNavigatorPtr createSpecific(const RapidXmlReaderNode&, const AronNavigatorPath&) const = 0;

                    static std::vector<std::pair<types::AronObjectTypeNavigatorPtr, std::vector<types::AronObjectTypeNavigatorPtr>>> GetAllProducedTopLevelObjects();

                    static void AddTopLevelObjectNames(const std::vector<std::string>&);
                    static void AddTopLevelObjectName(const std::string&);

                protected:
                    static std::string CurrentTopLevelObjectName;
                    static std::vector<std::pair<std::string, std::vector<std::string>>> AllTopLevelObjectNames;
                    static std::vector<std::string> AllLocalObjectNames;
                    static std::vector<std::pair<types::AronObjectTypeNavigatorPtr, std::vector<types::AronObjectTypeNavigatorPtr>>> AllTopLevelProducedObjects;
                    static std::vector<types::AronObjectTypeNavigatorPtr> AllLocalProducedObjects;

                private:
                    static const std::map<std::string, AronAbstractTypeXMLReaderFactoryPtr> Factories;
                };

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    class Aron##upperType##TypeXMLReaderFactory : \
        virtual public AronAbstractTypeXMLReaderFactory \
    { \
    public: \
        Aron##upperType##TypeXMLReaderFactory() = default; \
        virtual types::AronAbstractTypeNavigatorPtr createSpecific(const RapidXmlReaderNode&, const AronNavigatorPath&) const override; \
    };

                HANDLE_CONTAINER_TYPES
                HANDLE_EXTERNAL_TYPES
                HANDLE_COMPLEX_TYPES
#undef RUN_ARON_MACRO


                // Primitives
                template<typename AronAbstractTypeType, typename AronDataType, typename CppType> //requires AreAronAbstractTypeAndPrimitiveDependent<AronType, AronDataType, CppType>
                class AronPrimitiveTypeXMLReaderFactory :
                    virtual public AronAbstractTypeXMLReaderFactory
                {
                public:
                    AronPrimitiveTypeXMLReaderFactory() = default;

                    virtual types::AronAbstractTypeNavigatorPtr createSpecific(const RapidXmlReaderNode&, const AronNavigatorPath& path) const override
                    {
                        return types::AronAbstractTypeNavigatorPtr(new types::AronPrimitiveTypeNavigator<AronAbstractTypeType, AronDataType, CppType>(path));
                    }
                };
            }
        }
    }
}
