/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "AronBlobDataNavigator.h"

// ArmarX
#include <ArmarXCore/core/exceptions/Exception.h>
#include <RobotAPI/libraries/aron/data/AronDataNavigatorFactory.h>



namespace armarx
{
    namespace aron
    {
        namespace data
        {
            // constructors
            AronBlobDataNavigator::AronBlobDataNavigator(const AronNavigatorPath& path) :
                AronNavigator<AronDataDescriptor, AronData>::AronNavigator(AronDataDescriptor::eAronBlobData),
                AronDataNavigator(AronDataDescriptor::eAronBlobData, path),
                aron(new AronBlob())
            {

            }

            AronBlobDataNavigator::AronBlobDataNavigator(const AronBlobPtr& o, const AronNavigatorPath& path) :
                AronNavigator<AronDataDescriptor, AronData>::AronNavigator(AronDataDescriptor::eAronBlobData),
                AronDataNavigator(AronDataDescriptor::eAronBlobData, path),
                aron(o)
            {

            }

            // static methods
            AronBlobDataNavigatorPtr AronBlobDataNavigator::DynamicCast(const AronDataNavigatorPtr& n)
            {
                AronBlobDataNavigatorPtr casted = std::dynamic_pointer_cast<AronBlobDataNavigator>(n);
                checkDataNavigatorPtrForNull(casted, "");
                return casted;
            }

            // public member functions
            unsigned char* AronBlobDataNavigator::getData() const
            {
                return aron->data.data();
            }

            void AronBlobDataNavigator::setData(unsigned int elements, const unsigned char* src)
            {
                aron->data = std::vector<unsigned char>(elements);
                memcpy(aron->data.data(), src, elements);
            }

            // virtual implementations
            AronDataPtr AronBlobDataNavigator::getResult() const
            {
                return aron;
            }

            std::vector<AronDataNavigatorPtr> AronBlobDataNavigator::getChildren() const
            {
                return {};
            }

            size_t AronBlobDataNavigator::childrenSize() const
            {
                return 0;
            }
        }
    }
}
