/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <map>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/data/AronDataNavigator.h>

namespace armarx
{
    namespace aron
    {
        namespace data
        {
            class AronBlobDataNavigator;
            typedef std::shared_ptr<AronBlobDataNavigator> AronBlobDataNavigatorPtr;

            class AronBlobDataNavigator :
                virtual public AronDataNavigator
            {
            public:
                using PointerType = AronBlobDataNavigatorPtr;

            public:
                // constructors
                AronBlobDataNavigator(const AronNavigatorPath& path);
                AronBlobDataNavigator(const AronBlobPtr&, const AronNavigatorPath& path);

                // static methods
                static AronBlobDataNavigatorPtr DynamicCast(const AronDataNavigatorPtr& n);

                // public member functions
                unsigned char* getData() const;
                void setData(unsigned int, const unsigned char*);

                // virtual implementations
                virtual AronDataPtr getResult() const override;
                virtual std::vector<AronDataNavigatorPtr> getChildren() const override;
                virtual size_t childrenSize() const override;

            private:
                AronBlobPtr aron;
            };
        }
    }
}
