/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "AronDataNavigator.h"

// ArmarX
#include <ArmarXCore/core/exceptions/Exception.h>

#include <RobotAPI/libraries/aron/data/AronDataNavigatorFactory.h>

namespace armarx
{
    namespace aron
    {
        namespace data
        {
            // static data members
            const AronDataNavigatorFactoryPtr AronDataNavigator::Factory = AronDataNavigatorFactoryPtr(new AronDataNavigatorFactory());

            // constructors
            AronDataNavigator::AronDataNavigator(const AronDataDescriptor& descriptor, const AronNavigatorPath& path) :
                AronNavigator<AronDataDescriptor, AronData>::AronNavigator(descriptor),
                path(path)
            {
            }

            // static methods
            AronDataNavigatorPtr AronDataNavigator::FromAronPtr(const AronDataPtr& a, const AronNavigatorPath& path)
            {
                return Factory->create(a, path);
            }

            void AronDataNavigator::checkDataNavigatorPtrForNull(const AronDataNavigatorPtr& data, const std::string& message)
            {
                if (data.get() == nullptr)
                {
                    throw LocalException("Could not cast an AronDataNavigatorPtr. The Ptr was NULL. " + message);
                }
            }
        }
    }
}
