/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <unordered_map>

// ArmarX
#include <ArmarXCore/core/exceptions/Exception.h>
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/AronNavigator.h>
#include <RobotAPI/libraries/aron/AronNavigatorPath.h>

namespace armarx
{
    namespace aron
    {
        namespace data
        {
            class AronDataNavigatorFactory;
            typedef std::shared_ptr<AronDataNavigatorFactory> AronDataNavigatorFactoryPtr;

            class AronDataNavigator;
            typedef std::shared_ptr<AronDataNavigator> AronDataNavigatorPtr;

            class AronDataNavigator :
                virtual public AronNavigator<AronDataDescriptor, AronData>
            {
            public:
                using PointerType = AronDataNavigatorPtr;

            public:
                // constructors
                AronDataNavigator(const AronDataDescriptor&, const AronNavigatorPath&);

                // static methods
                static AronDataNavigatorPtr FromAronPtr(const AronDataPtr&, const AronNavigatorPath&);

                template<typename AronNavigatorType>
                static typename AronNavigatorType::PointerType DynamicCast(const AronDataNavigatorPtr& n)
                {
                    return AronNavigatorType::DynamicCast(n);
                }

                // virtual definitions of base class
                virtual AronDataPtr getResult() const override = 0;

                // virtual definitions
                virtual std::vector<AronDataNavigatorPtr> getChildren() const = 0;
                virtual size_t childrenSize() const = 0;

                // Getter/Setter
                AronNavigatorPath getPath() const
                {
                    return path;
                }
                std::string pathToString() const
                {
                    return path.toString();
                }

            protected:
                static void checkDataNavigatorPtrForNull(const AronDataNavigatorPtr& data, const std::string& message);

            private:
                AronNavigatorPath path;
                static const AronDataNavigatorFactoryPtr Factory;
            };
        }
    }
}
