/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <unordered_map>

// ArmarX
#include <RobotAPI/libraries/aron/AronFactory.h>
#include <RobotAPI/libraries/aron/data/AronDataNavigator.h>
#include <RobotAPI/libraries/aron/data/AronListDataNavigator.h>
#include <RobotAPI/libraries/aron/data/AronDictDataNavigator.h>
#include <RobotAPI/libraries/aron/data/AronBlobDataNavigator.h>
#include <RobotAPI/libraries/aron/data/AronPrimitiveDataNavigator.h>

#include <RobotAPI/libraries/aron/AronDescriptor.h>

namespace armarx
{
    namespace aron
    {
        namespace data
        {
            class AronDataNavigatorFactory;
            typedef std::shared_ptr<AronDataNavigatorFactory> AronDataNavigatorFactoryPtr;

            class AronDataNavigatorFactory :
                virtual public AronFactory<AronDataPtr, AronDataNavigatorPtr>
            {
            public:
                AronDataNavigatorFactory() = default;
                virtual AronDataNavigatorPtr create(const AronDataPtr&, const AronNavigatorPath&) const override;
                virtual AronDataNavigatorPtr createSpecific(const AronDataPtr&, const AronNavigatorPath&) const override;

            private:
                static const std::map<AronDataDescriptor, AronDataNavigatorFactoryPtr> Factories;
            };

            // Container Factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    class Aron##upperType##DataNavigatorFactory : \
        virtual public AronDataNavigatorFactory \
    { \
    public: \
        Aron##upperType##DataNavigatorFactory() = default; \
        virtual AronDataNavigatorPtr createSpecific(const AronDataPtr&, const AronNavigatorPath&) const override; \
    };

            HANDLE_CONTAINER_DATA
#undef RUN_ARON_MACRO

            // Complex factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    class Aron##upperType##DataNavigatorFactory : \
        virtual public AronDataNavigatorFactory \
    { \
    public: \
        Aron##upperType##DataNavigatorFactory() = default; \
        virtual AronDataNavigatorPtr createSpecific(const AronDataPtr&, const AronNavigatorPath&) const override; \
    };

            HANDLE_COMPLEX_DATA
#undef RUN_ARON_MACRO

            // Primitive Factories
            template<typename AronDataType, typename CppType> //requires AreAronPrimitiveDependent<AronDataType, CppType>
            class AronPrimitiveDataNavigatorFactory :
                virtual public AronDataNavigatorFactory
            {
            public:
                AronPrimitiveDataNavigatorFactory() = default;
                virtual AronDataNavigatorPtr createSpecific(const AronDataPtr& aron, const AronNavigatorPath& path) const override
                {
                    typename AronDataType::PointerType aronPrimitive = AronDataType::PointerType::dynamicCast(aron);
                    return AronPrimitiveDataNavigatorPtr<AronDataType, CppType>(new AronPrimitiveDataNavigator<AronDataType, CppType>(aronPrimitive, path));
                }
            };
        }
    }
}
