/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/data/AronDataNavigator.h>

namespace armarx
{
    namespace aron
    {
        namespace data
        {
            template<typename AronDataType, typename CppType> //requires AreAronPrimitiveDependent<AronDataType, CppType>
            class AronPrimitiveDataNavigator;

            template<typename AronDataType, typename CppType> //requires AreAronPrimitiveDependent<AronDataType, CppType>
            using AronPrimitiveDataNavigatorPtr = std::shared_ptr<AronPrimitiveDataNavigator<AronDataType, CppType>>;


            template<typename AronDataType, typename CppType> //requires AreAronPrimitiveDependent<AronDataType, CppType>
            class AronPrimitiveDataNavigator :
                virtual public AronDataNavigator
            {
            public:
                using PointerType = AronPrimitiveDataNavigatorPtr<AronDataType, CppType>;

            public:
                // constructors
                AronPrimitiveDataNavigator(const AronNavigatorPath& path):
                    AronNavigator<AronDataDescriptor, AronData>::AronNavigator(AronResolver::GetTypeForAronDataTypeId(typeid(AronDataType))),
                    AronDataNavigator(AronResolver::GetTypeForAronDataTypeId(typeid(AronDataType)), path),
                    aron(new AronDataType())
                {
                }

                AronPrimitiveDataNavigator(const typename AronDataType::PointerType& p, const AronNavigatorPath& path) :
                    AronNavigator<AronDataDescriptor, AronData>::AronNavigator(AronResolver::GetTypeForAronDataTypeId(typeid(AronDataType))),
                    AronDataNavigator(AronResolver::GetTypeForAronDataTypeId(typeid(AronDataType)), path),
                    aron(p)
                {
                    checkAronPtrForNull(aron, pathToString());
                }

                // static methods
                static AronPrimitiveDataNavigatorPtr<AronDataType, CppType> DynamicCast(const AronDataNavigatorPtr& n)
                {
                    AronPrimitiveDataNavigatorPtr<AronDataType, CppType> casted = std::dynamic_pointer_cast<AronPrimitiveDataNavigator<AronDataType, CppType>>(n);
                    checkDataNavigatorPtrForNull(casted, "");
                    return casted;
                }

                // public member functions
                void setValue(const CppType& x)
                {
                    aron->value = x;
                }

                CppType getValue() const
                {
                    return aron->value;
                }

                // virtual implementations
                virtual AronDataPtr getResult() const override
                {
                    return aron;
                }

                virtual std::vector<AronDataNavigatorPtr> getChildren() const override
                {
                    return {};
                }

                virtual size_t childrenSize() const override
                {
                    return 0;
                }

            private:
                // members
                typename AronDataType::PointerType aron;
            };
        }
    }
}
