/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// ArmarX
#include <RobotAPI/libraries/aron/AronConfig.h>
#include <RobotAPI/libraries/aron/data/AronListDataNavigator.h>
#include <RobotAPI/libraries/aron/data/AronDictDataNavigator.h>
#include <RobotAPI/libraries/aron/data/AronBlobDataNavigator.h>

namespace armarx
{
    namespace aron
    {
        namespace io
        {
            class AronDataNavigatorWriterToken;
            typedef std::shared_ptr<AronDataNavigatorWriterToken> AronDataNavigatorWriterTokenPtr;

            class AronDataNavigatorWriterToken
            {
            public:
                using PointerType = AronDataNavigatorWriterTokenPtr;

            public:
                // constructor
                AronDataNavigatorWriterToken(const data::AronDataNavigatorPtr&);

                // public member functions
                AronDataDescriptor getDescriptor() const;
                void addElement(const data::AronDataNavigatorPtr&);
                data::AronDataNavigatorPtr getData() const;
                std::string getAccessor() const;

                // List and Tuple member functions
                unsigned int getCurrentIndex() const;

                // Dict and object member functions
                void setCurrentKey(const std::string&);
                std::string getCurrentKey() const;

            private:
                // members
                data::AronDataNavigatorPtr data;

                // list index
                unsigned int currentIndex;

                // dict members
                std::string currentKey;
            };
        }
    }
}
