/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

#include <memory>
#include <stack>
#include <sstream>

#include <RobotAPI/libraries/aron/io/AronWriter.h>

namespace armarx
{
    namespace aron
    {
        namespace io
        {
            class AronJSONWriter :
                virtual public AronWriter
            {
            protected:
                virtual void startObject() override;
                virtual void writeObjectBody(const AronObjectNavigatorPtr&) override;
                virtual void endObject() override;

                virtual void startList() override;
                virtual void writeListBody(const AronListNavigatorPtr&) override;
                virtual void endList() override;

                virtual void startInt() override;
                virtual void writeIntBody(const AronPrimitiveNavigatorPtr<AronInt, int>&);
                virtual void endInt() override;

                virtual void startFloat() override;
                virtual void writeFloatBody(const AronPrimitiveNavigatorPtr<AronFloat, float>&) override;
                virtual void endFloat() override;

                virtual void startDouble() override;
                virtual void writeDoubleBody(const AronPrimitiveNavigatorPtr<AronDouble, double>&) override;
                virtual void endDouble() override;

                virtual void startString() override;
                virtual void writeStringBody(const AronPrimitiveNavigatorPtr<AronString, std::string>&) override;
                virtual void endString() override;

                virtual void startBool() override;
                virtual void writeBoolBody(const AronPrimitiveNavigatorPtr<AronBool, bool>&) override;
                virtual void endBool() override;

                virtual void startEigenMatrix() override;
                virtual void writeEigenMatrixBody(const AronPrimitiveNavigatorPtr<AronEigenMatrix, Eigen::MatrixXf>&) override;
                virtual void endEigenMatrix() override;

            private:

            };
        }
    }
}
