/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// STD/STL

// Header
#include "AronAbstractTypeNavigatorFactory.h"

//ArmarX


namespace armarx
{
    namespace aron
    {
        namespace types
        {
            const std::map<AronTypeDescriptor, AronAbstractTypeNavigatorFactoryPtr> AronAbstractTypeNavigatorFactory::Factories =
            {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    {AronResolver::GetTypeForAronAbstractTypeId(typeid(Aron##upperType##Type)), AronAbstractTypeNavigatorFactoryPtr(new Aron##upperType##TypeNavigatorFactory())}, \

                HANDLE_CONTAINER_TYPES
#undef RUN_ARON_MACRO

                /*#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
                    {AronTypeResolver::GetTypeForAronAbstractTypeId(typeid(Aron##upperType##Type)), AronAbstractTypeNavigatorFactoryPtr(new Aron##upperType##TypeNavigatorFactory())}, \

                                HANDLE_COMPLEX_TYPES
                #undef RUN_ARON_MACRO*/

#define RUN_ARON_MACRO(cppType, upperType, lowerType, capsType) \
    {AronResolver::GetTypeForAronAbstractTypeId(typeid(Aron##upperType##Type)), AronAbstractTypeNavigatorFactoryPtr(new AronPrimitiveTypeNavigatorFactory<Aron##upperType##Type, Aron##upperType, cppType>())}, \

                HANDLE_PRIMITIVE_TYPES_WITH_CPP_TYPE
#undef RUN_ARON_MACRO
            };

            AronAbstractTypeNavigatorPtr AronAbstractTypeNavigatorFactory::create(const AronAbstractTypePtr& aron, const AronNavigatorPath& path) const
            {
                if (!aron)
                {
                    throw LocalException("AronAbstractTypeNavigatorFactory: An AronDataPtr is NULL! Cannot create navigator!");
                }

                auto factory_iterator = Factories.find(AronResolver::GetTypeForAronAbstractType(aron));
                if (factory_iterator == Factories.end())
                {
                    throw LocalException("AronAbstractTypeNavigatorFactory: Cannot find the desired factory. Cannot create navigator!");
                }
                return factory_iterator->second->createSpecific(aron, path);
            }

            AronAbstractTypeNavigatorPtr AronAbstractTypeNavigatorFactory::createSpecific(const AronAbstractTypePtr&, const AronNavigatorPath&) const
            {
                throw LocalException("AronAbstractTypeNavigatorFactory: Called disallowed method of an AronDataNavigatorFactory. Use child class instead!");
            }

            // Container factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    AronAbstractTypeNavigatorPtr  Aron##upperType##TypeNavigatorFactory::createSpecific(const AronAbstractTypePtr& aron, const AronNavigatorPath& path) const \
    { \
        Aron##upperType##TypePtr aronCastedType = Aron##upperType##TypePtr::dynamicCast(aron); \
        return AronAbstractTypeNavigatorPtr(new Aron##upperType##TypeNavigator(aronCastedType, path)); \
    }

            HANDLE_CONTAINER_TYPES
#undef RUN_ARON_MACRO

            // Complex factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    AronAbstractTypeNavigatorPtr  Aron##upperType##TypeNavigatorFactory::createSpecific(const AronAbstractTypePtr& aron, const AronNavigatorPath& path) const \
    { \
        Aron##upperType##TypePtr aronCastedType = Aron##upperType##TypePtr::dynamicCast(aron); \
        return AronAbstractTypeNavigatorPtr(new Aron##upperType##TypeNavigator(aronCastedType, path)); \
    }

            HANDLE_COMPLEX_TYPES
#undef RUN_ARON_MACRO
        }
    }
}




