/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <map>
#include <memory>

// ArmarXCore
#include <RobotAPI/libraries/aron/AronFactory.h>
#include <RobotAPI/libraries/aron/types/AronAllTypeNavigators.h>

#include <RobotAPI/libraries/aron/AronDescriptor.h>

namespace armarx
{
    namespace aron
    {
        namespace types
        {
            class AronAbstractTypeNavigatorFactory;
            typedef std::shared_ptr<AronAbstractTypeNavigatorFactory> AronAbstractTypeNavigatorFactoryPtr;

            class AronAbstractTypeNavigatorFactory :
                virtual public AronFactory<AronAbstractTypePtr, AronAbstractTypeNavigatorPtr>
            {
            public:
                AronAbstractTypeNavigatorFactory() = default;
                virtual AronAbstractTypeNavigatorPtr create(const AronAbstractTypePtr&, const AronNavigatorPath&) const override;
                virtual AronAbstractTypeNavigatorPtr createSpecific(const AronAbstractTypePtr&, const AronNavigatorPath&) const override;

            private:
                const static std::map<AronTypeDescriptor, AronAbstractTypeNavigatorFactoryPtr> Factories;
            };

            // Container factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    class Aron##upperType##TypeNavigatorFactory : \
        virtual public AronAbstractTypeNavigatorFactory \
    { \
    public: \
        Aron##upperType##TypeNavigatorFactory() = default; \
        virtual AronAbstractTypeNavigatorPtr createSpecific(const AronAbstractTypePtr&, const AronNavigatorPath&) const override; \
    };

            HANDLE_CONTAINER_TYPES
#undef RUN_ARON_MACRO

            // Complex factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    class Aron##upperType##TypeNavigatorFactory : \
        virtual public AronAbstractTypeNavigatorFactory \
    { \
    public: \
        Aron##upperType##TypeNavigatorFactory() = default; \
        virtual AronAbstractTypeNavigatorPtr createSpecific(const AronAbstractTypePtr&, const AronNavigatorPath&) const override; \
    };

            HANDLE_COMPLEX_TYPES
#undef RUN_ARON_MACRO

            // Primitive
            template<typename AronAbstractTypeType, typename AronDataType, typename CppType> //requires AreAronAbstractTypeAndPrimitiveDependent<AronType, AronDataType, CppType>
            class AronPrimitiveTypeNavigatorFactory :
                virtual public AronAbstractTypeNavigatorFactory
            {
            public:
                AronPrimitiveTypeNavigatorFactory() = default;
                virtual AronAbstractTypeNavigatorPtr createSpecific(const AronAbstractTypePtr&, const AronNavigatorPath& path) const override
                {
                    return AronPrimitiveTypeNavigatorPtr<AronAbstractTypeType, AronDataType, CppType>(new AronPrimitiveTypeNavigator<AronAbstractTypeType, AronDataType, CppType>(path));
                }
            };
        }
    }
}
