/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base Class
#include "AronAbstractTypeNavigator.h"

namespace armarx
{
    namespace aron
    {
        namespace types
        {

            class AronEigenMatrixTypeNavigator;
            typedef std::shared_ptr<AronEigenMatrixTypeNavigator> AronEigenMatrixTypeNavigatorPtr;

            class AronEigenMatrixTypeNavigator :
                virtual public AronAbstractTypeNavigator
            {
            public:
                using PointerType = AronEigenMatrixTypeNavigatorPtr;

            public:
                // constructors
                AronEigenMatrixTypeNavigator(const std::vector<int>&, const std::string&, const AronNavigatorPath& path);

                // static methods
                static AronEigenMatrixTypeNavigatorPtr DynamicCast(const AronAbstractTypeNavigatorPtr& n);

                // virtual implementations
                virtual AronAbstractTypePtr getResult() const override;

                virtual CppBlockPtr getInitializeAronTypeBlock(const std::string&) const override;
                virtual CppBlockPtr getInitializeMembersBlock(const std::string&) const override;
                virtual CppBlockPtr getResetBlock(const std::string&) const override;
                virtual CppBlockPtr getWriteBlock(const std::string&) const override;
                virtual CppBlockPtr getReadBlock(const std::string&) const override;
                virtual CppBlockPtr getEqualsBlock(const std::string&, const std::string&) const override;

            private:
                // members
                static const std::set<std::string> acceptedTypes;
                std::string usedType;
                std::vector<int> dimensions;
                AronEigenMatrixTypePtr type;
            };
        }
    }
}
