/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Boost
#include <boost/algorithm/string/replace.hpp>

// Header
#include "AronExternalTypeNavigator.h"

namespace armarx
{
    namespace aron
    {
        namespace types
        {
            // constructors
            AronExternalTypeNavigator::AronExternalTypeNavigator(const std::string& name, const AronNavigatorPath& path) :
                AronNavigator<AronTypeDescriptor, AronAbstractType>::AronNavigator(AronTypeDescriptor::eAronListType),
                AronAbstractTypeNavigator(AronTypeDescriptor::eAronExternalType, name, boost::core::demangle(typeid(AronData).name()), boost::core::demangle(typeid(AronExternalType).name())),
                type(new AronExternalType())
            {
            }

            // static methods
            AronExternalTypeNavigatorPtr AronExternalTypeNavigator::DynamicCast(const AronAbstractTypeNavigatorPtr& n)
            {
                AronExternalTypeNavigatorPtr casted = std::dynamic_pointer_cast<AronExternalTypeNavigator>(n);
                checkAbstractTypeNavigatorPtrForNull(casted, "");
                return casted;
            }

            // virtual implementations
            AronAbstractTypePtr AronExternalTypeNavigator::getResult() const
            {
                throw LocalException("AronExternalTypeNavigator: Called invalid function getResult. Nothing to get!");
                return type;
            }

            CppBlockPtr AronExternalTypeNavigator::getResetBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                b->addLine(accessor + ".reset();");
                return b;
            }

            CppBlockPtr AronExternalTypeNavigator::getInitializeMembersBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                b->addLine(accessor + ".initialize();");
                return b;
            }

            CppBlockPtr AronExternalTypeNavigator::getInitializeAronTypeBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                b->addLine(getFullAronTypePtrTypename() + " " + accessor + " = " + getFullAronTypePtrTypename() + "(new " + getFullAronTypeTypename() + "());");
                return b;
            }

            CppBlockPtr AronExternalTypeNavigator::getWriteBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                b->addLine(accessor + ".write(w);");
                return b;
            }

            CppBlockPtr AronExternalTypeNavigator::getReadBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                b->addLine(accessor + ".read(w);");
                return b;
            }

            CppBlockPtr AronExternalTypeNavigator::getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                b->addLine("if (not (" + accessor + " == " + otherInstanceAccessor + "))");
                b->addLine("\t return false;");
                return b;
            }
        }
    }
}
