/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Boost
#include <boost/algorithm/string.hpp>

// Header
#include "AronOpenCVMatTypeNavigator.h"

namespace armarx
{
    namespace aron
    {
        namespace types
        {

            const std::set<std::string> AronOpenCVMatTypeNavigator::acceptedTypes =
            {
                "",
                "",
                "",
                ""
            };

            // constructors
            AronOpenCVMatTypeNavigator::AronOpenCVMatTypeNavigator(const std::vector<int>& dims, const std::string& t, const AronNavigatorPath& path) :
                AronNavigator<AronTypeDescriptor, AronAbstractType>::AronNavigator(AronTypeDescriptor::eAronOpenCVMatType),
                AronAbstractTypeNavigator(AronTypeDescriptor::eAronOpenCVMatType, "cv::Mat", boost::core::demangle(typeid(AronBlob).name()), boost::core::demangle(typeid(AronOpenCVMatType).name())),
                dimensions(dims),
                type(new AronOpenCVMatType())
            {
                type->dimensions = dims;
                type->typeName = t;

                bool found = false;
                for (const auto& typeName : acceptedTypes)
                {
                    if (typeName == t)
                    {
                        this->usedType = typeName;
                        found = true;
                        break;
                    }
                }

                if (!found)
                {
                    throw LocalException("AronEigenTypeNavigator: Could not find the typename '" + t + "' in accepted types.");
                }
            }

            // static methods
            AronOpenCVMatTypeNavigatorPtr AronOpenCVMatTypeNavigator::DynamicCast(const AronAbstractTypeNavigatorPtr& n)
            {
                AronOpenCVMatTypeNavigatorPtr casted = std::dynamic_pointer_cast<AronOpenCVMatTypeNavigator>(n);
                checkAbstractTypeNavigatorPtrForNull(casted, "");
                return casted;
            }

            // virtual implementations
            AronAbstractTypePtr AronOpenCVMatTypeNavigator::getResult() const
            {
                return type;
            }

            CppBlockPtr AronOpenCVMatTypeNavigator::getResetBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                b->addLine(accessor + " = " + getFullCppTypename() + "();");
                return b;
            }

            CppBlockPtr AronOpenCVMatTypeNavigator::getInitializeMembersBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                b->addLine(accessor + " = " + getFullCppTypename() + "();");
                return b;
            }

            CppBlockPtr AronOpenCVMatTypeNavigator::getInitializeAronTypeBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                b->addLine(getFullAronTypePtrTypename() + " " + accessor + " = " + getFullAronTypePtrTypename() + "(new " + getFullAronTypeTypename() + "());");
                return b;
            }

            CppBlockPtr AronOpenCVMatTypeNavigator::getWriteBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                b->addLine("w.writeBlob(reinterpret_cast<unsigned char*>(" + accessor + ".pixels));");
                return b;
            }

            CppBlockPtr AronOpenCVMatTypeNavigator::getReadBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                //
                //b->addLine(accessor + " = " + getFullCppTypename() + "({" + implodeDimensionVector(dimensions) + "}, " + usedType + ", reinterpret_cast<unsigned char*>(r.readBlob()), 0);");
                return b;
            }

            CppBlockPtr AronOpenCVMatTypeNavigator::getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                return b;
            }
        }
    }
}

