/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Boost
#include <boost/algorithm/string.hpp>

// Header
#include "AronPCLPointcloudTypeNavigator.h"

namespace armarx
{
    namespace aron
    {
        namespace types
        {

            const std::vector<std::string> AronPCLPointcloudTypeNavigator::acceptedTypes =
            {
                "pcl::PointXYZRGBL",
                "",
                "",
                ""
            };

            // constructors
            AronPCLPointcloudTypeNavigator::AronPCLPointcloudTypeNavigator(const unsigned int w, const unsigned int h, const std::string& t, const AronNavigatorPath& path) :
                AronNavigator<AronTypeDescriptor, AronAbstractType>::AronNavigator(AronTypeDescriptor::eAronPCLPointcloudType),
                AronAbstractTypeNavigator(AronTypeDescriptor::eAronPCLPointcloudType, "pcl::PointCloud<" + t + ">", boost::core::demangle(typeid(AronBlob).name()), boost::core::demangle(typeid(AronPCLPointcloudType).name())),
                width(w),
                height(h),
                type(new AronPCLPointcloudType())
            {
                type->typeName = t;
                type->width = w;
                type->height = h;

                bool found = false;
                for (const auto& typeName : acceptedTypes)
                {
                    if (typeName == t)
                    {
                        this->usedType = typeName;
                        found = true;
                        break;
                    }
                }

                if (!found)
                {
                    throw LocalException("AronEigenTypeNavigator: Could not find the typename '" + t + "' in accepted types.");
                }
            }

            // static methods
            AronPCLPointcloudTypeNavigatorPtr AronPCLPointcloudTypeNavigator::DynamicCast(const AronAbstractTypeNavigatorPtr& n)
            {
                AronPCLPointcloudTypeNavigatorPtr casted = std::dynamic_pointer_cast<AronPCLPointcloudTypeNavigator>(n);
                checkAbstractTypeNavigatorPtrForNull(casted, "");
                return casted;
            }

            // virtual implementations
            AronAbstractTypePtr AronPCLPointcloudTypeNavigator::getResult() const
            {
                return type;
            }

            CppBlockPtr AronPCLPointcloudTypeNavigator::getResetBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                return b;
            }

            CppBlockPtr AronPCLPointcloudTypeNavigator::getInitializeMembersBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                return b;
            }

            CppBlockPtr AronPCLPointcloudTypeNavigator::getInitializeAronTypeBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                return b;
            }

            CppBlockPtr AronPCLPointcloudTypeNavigator::getWriteBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                b->addLine("w.writeBlob(reinterpret_cast<unsigned char*>(" + accessor + ".points.data()));");
                return b;
            }

            CppBlockPtr AronPCLPointcloudTypeNavigator::getReadBlock(const std::string& accessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                //PointCloud (uint32_t width_, uint32_t height_, const PointT& value_ = PointT ())
                b->addLine(accessor + " = " + getFullCppTypename() + "(" + std::to_string(width) + ", " + std::to_string(height) + ", " + usedType + ");");
                b->addLine("memcpy(" + accessor + ".points.data(), reinterpret_cast<" + usedType + "*>(r.readBlob()), " + accessor + ".width * " + accessor + ".height);");
                return b;
            }

            CppBlockPtr AronPCLPointcloudTypeNavigator::getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const
            {
                CppBlockPtr b = CppBlockPtr(new CppBlock());
                return b;
            }
        }
    }
}

