/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>

// Base Class
#include "AronAbstractTypeNavigator.h"

// ArmarX
#include <RobotAPI/libraries/aron/AronConcepts.h>

namespace armarx
{
    namespace aron
    {
        namespace types
        {
            template<typename AronAbstractTypeType, typename AronDataType, typename CppType> //requires AreAronAbstractTypeAndPrimitiveDependent<AronType, AronDataType, CppType>
            class AronPrimitiveTypeNavigator;

            template<typename AronAbstractTypeType, typename AronDataType, typename CppType> //requires AreAronAbstractTypeAndPrimitiveDependent<AronType, AronDataType, CppType>
            using AronPrimitiveTypeNavigatorPtr = std::shared_ptr<AronPrimitiveTypeNavigator<AronAbstractTypeType, AronDataType, CppType>>;

            template<typename AronAbstractTypeType, typename AronDataType, typename CppType> //requires AreAronAbstractTypeAndPrimitiveDependent<AronType, AronDataType, CppType>
            class AronPrimitiveTypeNavigator :
                virtual public AronAbstractTypeNavigator
            {
            public:
                using PointerType = AronPrimitiveTypeNavigatorPtr<AronAbstractTypeType, AronDataType, CppType>;

            public:
                // constructors
                AronPrimitiveTypeNavigator(const AronNavigatorPath& path) :
                    AronNavigator<AronTypeDescriptor, AronAbstractType>::AronNavigator(AronResolver::GetTypeForAronAbstractTypeId(typeid(AronAbstractTypeType))),
                    AronAbstractTypeNavigator(AronResolver::GetTypeForAronAbstractTypeId(typeid(AronAbstractTypeType)), boost::core::demangle(typeid(CppType).name()), boost::core::demangle(typeid(AronDataType).name()), boost::core::demangle(typeid(AronAbstractTypeType).name())),
                    type(new AronAbstractTypeType())
                {

                }

                // static methods
                AronPrimitiveTypeNavigatorPtr<AronAbstractTypeType, AronDataType, CppType> DynamicCast(const AronAbstractTypeNavigatorPtr& n)
                {
                    AronPrimitiveTypeNavigatorPtr<AronAbstractTypeType, AronDataType, CppType> casted = std::dynamic_pointer_cast<AronPrimitiveTypeNavigator<AronAbstractTypeType, AronDataType, CppType>>(n);
                    checkAbstractTypeNavigatorPtrForNull(casted, "");
                    return casted;
                }

                // virtual implementations
                virtual AronAbstractTypePtr getResult() const override
                {
                    return type;
                }

                virtual CppBlockPtr getResetBlock(const std::string& accessor) const override
                {
                    CppBlockPtr b = CppBlockPtr(new CppBlock());
                    b->addLine(accessor + " = {};");
                    return b;
                }

                virtual CppBlockPtr getInitializeMembersBlock(const std::string& accessor) const override
                {
                    CppBlockPtr b = CppBlockPtr(new CppBlock());
                    b->addLine(accessor + " = {};");
                    return b;
                }

                virtual CppBlockPtr getInitializeAronTypeBlock(const std::string& accessor) const override
                {
                    CppBlockPtr b = CppBlockPtr(new CppBlock());
                    b->addLine(getFullAronTypePtrTypename() + " " + accessor + " = " + getFullAronTypePtrTypename() + "(new " + getFullAronTypeTypename() + "());");
                    return b;
                }

                virtual CppBlockPtr getWriteBlock(const std::string& accessor) const override
                {
                    CppBlockPtr b = CppBlockPtr(new CppBlock());
                    b->addLine("w.write" + getCppTypename() + "(" + accessor + ");");
                    return b;
                }

                virtual CppBlockPtr getReadBlock(const std::string& accessor) const override
                {
                    CppBlockPtr b = CppBlockPtr(new CppBlock());
                    b->addLine(accessor + " = r.read" + getCppTypename() + "();");
                    return b;
                }

                CppBlockPtr getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const override
                {
                    CppBlockPtr b = CppBlockPtr(new CppBlock());
                    b->addLine("if (not (" + accessor + " == " + otherInstanceAccessor + "))");
                    b->addLine("\t return false;");
                    return b;
                }

            private:
                // members
                typename AronAbstractTypeType::PointerType type;
            };
        }
    }
}
